--[[
--
-- oUF_Rabbit
--
-- Custom layout for oUF.
-- Based on VuBce, rUnits, TsoHG and Lily.
-- by rabbit.magtheridon@gmail.com
--
--]]

local statusbarTexture = "Interface\\AddOns\\oUF_Rabbit\\media\\armory"
local statusbarTextureVertical = "Interface\\AddOns\\oUF_Rabbit\\media\\armory-vertical"
local borderTexture = "Interface\\AddOns\\oUF_Rabbit\\media\\border"
local font = "Interface\\AddOns\\oUF_Rabbit\\media\\font.ttf"
local power = oUF.colors.power

--local raidHpColor = { 0.55, 0.7, 0.6 }
local raidHpColor = { 115 / 255, 191 / 255, 96 / 255 }

local function menu(self)
	local unit = self.unit:sub(1, -2)
	local cunit = self.unit:gsub("^%l", string.upper)

	if unit == "party" or unit == "partypet" then
		ToggleDropDownMenu(1, nil, _G["PartyMemberFrame"..self.id.."DropDown"], "cursor", 0, 0)
	elseif _G[cunit.."FrameDropDown"] then
		ToggleDropDownMenu(1, nil, _G[cunit.."FrameDropDown"], "cursor", 0, 0)
	end
end

local percent = "%d%%"
local dead = "DEAD"
local ghost = "GHOST"
local offline = "OFF"

local function updateNameRaid(self, event, unit)
	if self.unit == unit or (not unit and self.unit) then
		local textFrame = self.Name
		local u = unit or self.unit
		if UnitIsDead(u) then
			textFrame:SetText(dead)
			self:SetBackdropColor(0, 0, 0, 0)
		elseif UnitIsGhost(u) then
			textFrame:SetText(ghost)
			self:SetBackdropColor(0, 0, 0, 0)
		elseif not UnitIsConnected(u) then
			textFrame:SetText(offline)
			self:SetBackdropColor(0, 0, 0, 0)
		else
			local min, max = UnitHealth(u), UnitHealthMax(u)
			local p = floor(min / max * 100)
			if p < 90 then
				textFrame:SetText(p)
			else
				local n = UnitName(u)
				textFrame:SetText(n:sub(1, 3))
			end
			self:SetBackdropColor(0, 0, 0, .9)
		end
		local _, c = UnitClass(u)
		local color = RAID_CLASS_COLORS[c]
		if color then
			textFrame:SetTextColor(color.r, color.g, color.b)
		end
	end
end

local function updateHealthRaid(self, event, bar, unit, min, max)
	self:UNIT_NAME_UPDATE(event, unit)
end

local function updatePowerRaid(self, event, bar, unit, min, max)
	local p = UnitPowerType(unit)
	if p == 0 then
		self.Health:SetWidth(33)
		bar:Show()
	else
		self.Health:SetWidth(38)
		bar:Hide()
	end
end

local grey = {0.5, 0.5, 0.5}
local white = {1, 1, 1}
local function updateName(self, event, unit)
	if self.unit == unit or (not unit and self.unit) then
		local u = unit or self.unit
		local c = (not UnitIsConnected(u) or UnitIsGhost(u) or UnitIsDead(u)) and grey or white

		self.Name:SetText(UnitName(u))
		self.Name:SetTextColor(unpack(c))

		if self.Lvl then
			local lvlFrame = self.Lvl
			local level = UnitLevel(u)
			local cl = UnitClassification(u)

			if not cl or cl == "normal" then
				lvlFrame:SetText(level)
			elseif cl == "worldboss" then
				lvlFrame:SetText("Boss")
			elseif cl == "rareelite" then
				lvlFrame:SetText(level .. "+ Rare")
			elseif cl == "elite" then
				lvlFrame:SetText(level .. "+")
			elseif cl == "rare" then
				lvlFrame:SetText(level .. " Rare")
			else
				lvlFrame:SetText(level)
			end
			lvlFrame:SetTextColor(unpack(c))
		end

		if self.Class then
			local classFrame = self.Class
			if UnitIsPlayer(u) then
				local c, eC = UnitClass(u)
				classFrame:SetText(c)
				local color = RAID_CLASS_COLORS[eC]
				if color then
					classFrame:SetTextColor(color.r, color.g, color.b)
				end
			else
				classFrame:SetTextColor(unpack(c))
				classFrame:SetText(UnitCreatureFamily(u) or UnitCreatureType(u))
			end
		end

		if self.Race then
			self.Race:SetText(UnitRace(u))
			self.Race:SetTextColor(unpack(c))
		end
	end
end

local function updateHealth(self, event, bar, unit, min, max)
	if UnitIsDead(unit) then
		bar.value:SetText("Dead")
		bar.value:SetVertexColor(.5, .5, .5)
	elseif UnitIsGhost(unit) then
		bar.value:SetText("Ghost")
		bar.value:SetVertexColor(.5, .5, .5)
	elseif not UnitIsConnected(unit) then
		bar.value:SetText("Off")
		bar.value:SetVertexColor(.5, .5, .5)
	else
		local value = nil
		if max == 100 then -- Percentage value
			value = min .. "%"
		elseif min ~= max then
			value = "-" .. (max - min)
		elseif unit == "target" then
			value = max
		end
		if value then
			bar.value:SetText(value)
			if max == min then
				bar.value:SetVertexColor(1, 1, 1)
			else
				bar.value:SetVertexColor(1, 0.5, 0.5)
			end
		else
			bar.value:SetText("")
		end
		if UnitIsTapped(unit) and not UnitIsTappedByPlayer(unit) then
			bar:SetStatusBarColor(.5, .5, .3)
		else
			local _, c = UnitClass(unit)
			local color = UnitIsFriend(unit, "player") and UnitIsPlayer(unit) and RAID_CLASS_COLORS[c] or UnitReactionColor[UnitReaction(unit, "player")]
			if color then
				bar:SetStatusBarColor(color.r, color.g, color.b)
			end
			self:UNIT_NAME_UPDATE(event, unit)
		end
	end
end

local function updatePower(self, event, bar, unit, min, max)
	if UnitIsTapped(unit) and not UnitIsTappedByPlayer(unit) or not UnitIsConnected(unit) then
		bar:SetStatusBarColor(.6, .6, .6)
	else
		local color = power[UnitPowerType(unit)]
		if color then
			bar:SetStatusBarColor(color.r, color.g, color.b)
		end
	end
end

local function auraIcon(self, button)
	local t = button:CreateTexture(nil, "OVERLAY")
	t:SetTexture(borderTexture)
	t:SetAllPoints(button)
	t:SetVertexColor(0.25, 0.25, 0.35)
end

local backdrop = {
	bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", 
	insets = {left = -2, right = -2, top = -2, bottom = -2},
}

local function func(settings, self, unit)
	self.menu = menu
	self:EnableMouse(true)
	self:SetScript("OnEnter", UnitFrame_OnEnter)
	self:SetScript("OnLeave", UnitFrame_OnLeave)
	self:RegisterForClicks("anyup")
	self:SetAttribute("*type2", "menu")

	if not unit then -- Party
		self:SetWidth(168) 
		self:SetHeight(36)
	elseif unit == "targettarget" or unit == "focus" then
		self:SetWidth(92)
		self:SetHeight(20)
	elseif unit == "player" or unit == "target" then
		self:SetWidth(192)
		self:SetHeight(36)
	elseif unit == "pet" then
		self:SetWidth(192)
		self:SetHeight(20)
	end

	self:SetBackdrop(backdrop)
	self:SetBackdropColor(0, 0, 0, .9)

	local hp = CreateFrame("StatusBar")
	hp:SetHeight(20)
	hp:SetStatusBarTexture(statusbarTexture)
	hp:SetStatusBarColor(0, 1, 0)
	hp:SetParent(self)
	hp:SetPoint("TOP")
	hp:SetPoint("LEFT")
	hp:SetPoint("RIGHT")

	local hpp = hp:CreateFontString(nil, "OVERLAY")
	hpp:SetPoint("RIGHT", -4, 0)
	hpp:SetFontObject(GameFontNormal)
	hpp:SetVertexColor(1, 0.5, 0.5)

	hp.value = hpp
	self.Health = hp
	self.OverrideUpdateHealth = updateHealth

	local name = hp:CreateFontString(nil, "OVERLAY")
	name:SetPoint("LEFT", 4, 0)
	name:SetPoint("RIGHT", hpp, "LEFT")
	name:SetJustifyH("LEFT")
	name:SetFont(font, 16)
	name:SetShadowColor(0, 0, 0, 0.9)
	name:SetShadowOffset(1, -1)
	name:SetVertexColor(1, 1, 1)
	self.Name = name
	self.UNIT_NAME_UPDATE = updateName

	if unit == "player" then
		local cbft = hp:CreateFontString(nil, "OVERLAY")
		cbft:SetPoint("CENTER", self, "CENTER")
		cbft:SetFontObject(GameFontNormal)
		self.CombatFeedbackText = cbft
		self.CombatFeedbackText.maxAlpha = .6
		-- The only thing we don't ignore is heals.
		self.CombatFeedbackText.ignoreImmune = true
		self.CombatFeedbackText.ignoreDamage = true
		self.CombatFeedbackText.ignoreEnergize = true
		self.CombatFeedbackText.ignoreOther = true
	end

	local ricon = hp:CreateTexture(nil, "OVERLAY")
	ricon:SetHeight(24)
	ricon:SetWidth(24)
	ricon:SetPoint("CENTER", hp, "TOP", 0, 0)
	ricon:SetTexture("Interface\\TargetingFrame\\UI-RaidTargetingIcons")
	self.RaidIcon = ricon

	if unit ~= "targettarget" and unit ~= "focus" and unit ~= "pet" then
		local pp = CreateFrame("StatusBar")
		pp:SetHeight(16)
		pp:SetStatusBarTexture(statusbarTexture)
		pp:SetStatusBarColor(.25, .25, .35)

		pp:SetParent(self)
		pp:SetPoint("BOTTOM", self, "BOTTOM")
		pp:SetPoint("LEFT")
		pp:SetPoint("RIGHT")

		self.Power = pp
		self.OverrideUpdatePower = updatePower

		if unit ~= "player" then
			local fontName, _, fontFlags = GameFontHighlight:GetFont()

			local lvl = pp:CreateFontString(nil, "OVERLAY")
			lvl:SetPoint("LEFT", pp, "LEFT", 4, 0)
			lvl:SetFontObject(GameFontHighlight)
			lvl:SetFont(fontName, 11, fontFlags)
			self.Lvl = lvl

			local class = pp:CreateFontString(nil, "OVERLAY")
			class:SetPoint("LEFT", lvl, "RIGHT",  4, 0)
			class:SetFontObject(GameFontHighlight)
			class:SetFont(fontName, 11, fontFlags)
			self.Class = class

			local race = pp:CreateFontString(nil, "OVERLAY")
			race:SetPoint("LEFT", class, "RIGHT",  4, 0)
			race:SetFontObject(GameFontHighlight)
			race:SetFont(fontName, 11, fontFlags)
			self.Race = race

			local leader = hp:CreateTexture(nil, "OVERLAY")
			leader:SetHeight(14)
			leader:SetWidth(14)
			leader:SetPoint("BOTTOMRIGHT", hp, "TOPLEFT", 7, -7)
			leader:SetTexture("Interface\\GroupFrame\\UI-Group-LeaderIcon")
			self.Leader = leader
		end
	end

	local buffs, debuffs = nil, nil
	if not unit then
		debuffs = CreateFrame("Frame", nil, self)
		local s = self.Power:GetHeight() + self.Health:GetHeight()
		debuffs.size = s
		debuffs:SetHeight(s)
		debuffs:SetWidth(s * 3)
		debuffs:SetPoint("LEFT", self, "RIGHT", 4, 0)
		debuffs.initialAnchor = "TOPLEFT"
		debuffs.num = 3 -- max debuffs
		self.PostCreateAuraIcon = auraIcon
	elseif unit == "target" or unit == "player" then
		buffs = CreateFrame("Frame", nil, self)
		buffs.size = self:GetWidth() / 8
		buffs.num = 8
		buffs:SetHeight(buffs.size)
		buffs:SetWidth(self:GetWidth())

		buffs:SetPoint("BOTTOMLEFT", self, "TOPLEFT", 0, 4)
		buffs.initialAnchor = "BOTTOMLEFT"
		buffs["growth-x"] = "RIGHT"
		buffs["growth-y"] = "UP"
		buffs.filter = (unit == "player")

		debuffs = CreateFrame("Frame", nil, self)
		debuffs:SetPoint("BOTTOMLEFT", buffs, "TOPLEFT", 0, 2)
		debuffs:SetPoint("BOTTOMRIGHT", buffs, "TOPRIGHT", 0, 2)
		debuffs.size = self:GetWidth() / 8
		debuffs.num = 16
		debuffs:SetHeight(debuffs.size * 2)

		debuffs.initialAnchor = "BOTTOMRIGHT"
		debuffs["growth-x"] = "LEFT"
		debuffs["growth-y"] = "UP"
	end

	self.Buffs = buffs
	self.Debuffs = debuffs

	if unit == "target" then
		local cpoints = self:CreateFontString(nil, "OVERLAY")
		cpoints:SetPoint("RIGHT", self, "LEFT", -8, 1)
		cpoints:SetFont(DAMAGE_TEXT_FONT, 38, "OUTLINE")
		cpoints:SetVertexColor(1, 1, 1)
		cpoints:SetJustifyH("RIGHT")
		self.CPoints = cpoints
	end

	if not unit then
		self.Range = true
		self.inRangeAlpha = 1
		self.outsideRangeAlpha = .5
	end

	return self
end

local function raid(settings, self, unit)
	self.menu = menu

	self:EnableMouse(true)
	self:SetScript("OnEnter", UnitFrame_OnEnter)
	self:SetScript("OnLeave", UnitFrame_OnLeave)
	self:RegisterForClicks("anyup")
	self:SetAttribute("*type2", "menu")

	self:SetBackdrop(backdrop)
	self:SetBackdropColor(0, 0, 0, 1)

	local hp = CreateFrame("StatusBar")
	hp:SetStatusBarTexture(statusbarTextureVertical)
	hp:SetStatusBarColor(unpack(raidHpColor))
	hp:SetOrientation("VERTICAL")
	hp:SetParent(self)
	hp:SetPoint("LEFT", self, "LEFT")
	hp:SetPoint("TOP")
	hp:SetPoint("BOTTOM")
	hp:SetWidth(33)
	self.Health = hp
	self.OverrideUpdateHealth = updateHealthRaid

	local mana = power[0]
	local pp = CreateFrame("StatusBar")
	pp:SetStatusBarTexture(statusbarTextureVertical)
	pp:SetStatusBarColor(mana.r, mana.g, mana.b)
	pp:SetOrientation("VERTICAL")
	pp:SetParent(self)
	pp:SetPoint("RIGHT", self, "RIGHT")
	pp:SetPoint("TOP")
	pp:SetPoint("BOTTOM")
	pp:SetWidth(4)
	self.Power = pp
	self.OverrideUpdatePower = updatePowerRaid

	-- We need an overlay to span the name across both hp and power
	local overlay = CreateFrame("Button")
	overlay:SetParent(self)
	overlay:SetAllPoints(self)
	overlay:SetBackdropColor(0, 0, 0, 0)
	overlay:EnableMouse(false)

	local name = overlay:CreateFontString(nil, "OVERLAY")
	name:SetAllPoints(overlay)
	name:SetJustifyH("CENTER")
	name:SetFont(font, 13)
	name:SetShadowColor(0, 0, 0, 0.9)
	name:SetShadowOffset(1, -1)
	self.Name = name
	self.UNIT_NAME_UPDATE = updateNameRaid

	local icon = self:CreateTexture(nil, "OVERLAY")
	icon:SetHeight(16)
	icon:SetWidth(16)
	icon:SetPoint("TOP", self, 0, 8)
	icon:SetTexture("Interface\\TargetingFrame\\UI-RaidTargetingIcons")
	self.RaidIcon = icon

	local leader = self:CreateTexture(nil, "OVERLAY")
	leader:SetHeight(13)
	leader:SetWidth(13)
	leader:SetPoint("TOPLEFT", self, 3, 6)
	leader:SetTexture("Interface\\GroupFrame\\UI-Group-LeaderIcon")
	self.Leader = leader

	self.Range = true
	self.inRangeAlpha = 1
	self.outsideRangeAlpha = .5

	return self
end

oUF:RegisterStyle("Rabbit", setmetatable({}, {__call = func}))
oUF:RegisterStyle("Rabbit_Raid", setmetatable({
	["initial-width"] = 38,
	["initial-height"] = 24,
}, {__call = raid}))


oUF:SetActiveStyle("Rabbit")

local player = oUF:Spawn("player", "oUF_Player")
player:SetPoint("BOTTOM", -114, 160)
local pet = oUF:Spawn("pet", "oUF_Pet")
pet:SetPoint("TOPLEFT", player, "BOTTOMLEFT", 0, -8)

local target = oUF:Spawn("target", "oUF_Target")
target:SetPoint("BOTTOM", 114, 160)

local tot = oUF:Spawn("targettarget", "oUF_TargetTarget")
tot:SetPoint("TOPRIGHT", target, "BOTTOMRIGHT", 0, -8)
local focus = oUF:Spawn("focus", "oUF_Focus")
focus:SetPoint("TOPLEFT", target, "BOTTOMLEFT", 0, -8)

local party = oUF:Spawn("header", "oUF_Party")
party:SetPoint("TOPLEFT", 4, -40)
party:SetAttribute("yOffset", -8)
party:SetAttribute("showParty", true)
party:Show()

local toggleParty = CreateFrame("Frame")
toggleParty:SetScript("OnEvent", function()
	if GetNumRaidMembers() > 0 then
		party:Hide()
	elseif GetNumPartyMembers() > 0 then
		party:Show()
	end
end)
toggleParty:RegisterEvent("PARTY_MEMBERS_CHANGED")
toggleParty:RegisterEvent("PARTY_LEADER_CHANGED")
toggleParty:RegisterEvent("RAID_ROSTER_UPDATE")
toggleParty:RegisterEvent("PLAYER_LOGIN")

oUF:SetActiveStyle("Rabbit_Raid")

local raid = {}
for i = 1, 8 do
	local rGroup = oUF:Spawn("header", "oUF_Raid" .. i)
	rGroup:SetAttribute("groupFilter", tostring(i))
	rGroup:SetAttribute("showRaid", true)
	rGroup:SetAttribute("yOffset", -8)
	table.insert(raid, rGroup)
	if i == 1 then
		rGroup:SetPoint("TOPLEFT", UIParent, "TOPLEFT", 4, -40)
	else
		rGroup:SetPoint("TOPLEFT", raid[i-1], "TOPRIGHT", 8, 0)		
	end
	rGroup:Show()
end

